"""distutils

The main package for the Python Module Distribution Utilities.  Normally
used from a setup script as

   from distutils.core import setup

   setup (...)
"""

# This module should be kept compatible with Python 1.5.2.

__revision__ = "$Id: __init__.py,v 1.25 2004/07/23 19:47:32 theller Exp $"

__version__ = "2.4.0"
